﻿using UnityEngine;
using System.Collections;

public class ScreenController : MonoBehaviour
{
	private enum ScreenMoveState
	{
		None,
		TowardCamera,
		TowardOriginalPos
	};

	public Transform inFrontOfCameraTransform;
	public Transform screenTransform;
	public Camera screenCamera;
	public GameObject screenObject;
	public float screenWidth = 0.5f;
	public float screenHeight = 0.5f;

	private MouseLook[] mouseLooks;

	private bool screenIsUp;
	private ScreenMoveState moveState;
	private Vector3 screenStartLocalPos;
	private Quaternion screenStartLocalRot;

	void Awake()
	{
		screenIsUp = false;
		screenStartLocalPos = screenTransform.localPosition;
		screenStartLocalRot = screenTransform.localRotation;
		moveState = ScreenMoveState.None;

		//This ended up warping the texture's aspect ratio somehow... leaving it using a premade RenderTexture instead.
		//int theTextureHeight = (int)( Screen.height * 0.75f );

		//screenCamera.targetTexture = new RenderTexture( theTextureHeight, theTextureHeight, 24 );
		//screenObject.renderer.material.mainTexture = screenCamera.targetTexture;
	}

	void Start()
	{
		mouseLooks = GetComponentsInChildren<MouseLook>();
		Screen.lockCursor = true;
	}
	
	void Update()
	{
		HandleInputs();
		UpdateScreenMovement();
	}

	private void HandleInputs()
	{
		if( Input.GetButtonDown( "Toggle Screen" ) )
		{
			ToggleScreen();
		}
		
		if( screenIsUp )
		{
			if( Input.GetButtonDown( "Touch Screen" ) )
			{
				TouchScreen();
			}
			else
			{
				HoverScreen();
			}
		}
	}

	private void UpdateScreenMovement()
	{
		if( moveState != ScreenMoveState.None )
		{
			Vector3 theGoalLocalPos;
			Quaternion theGoalLocalRot;
			if( moveState == ScreenMoveState.TowardCamera )
			{
				theGoalLocalPos = Vector3.zero;
				theGoalLocalRot = Quaternion.identity;
			}
			else
			{
				theGoalLocalPos = screenStartLocalPos;
				theGoalLocalRot = screenStartLocalRot;
			}

			float theLerpAmount = Time.deltaTime * 10.0f;
			Vector3 theNewPos = Vector3.Lerp( screenTransform.localPosition, theGoalLocalPos, theLerpAmount );
			Quaternion theNewRot = Quaternion.Lerp( screenTransform.localRotation, theGoalLocalRot, theLerpAmount );

			if( Vector3.Distance( theNewPos, theGoalLocalPos ) < 0.01f )
			{
				screenTransform.localPosition = theGoalLocalPos;
				screenTransform.localRotation = theGoalLocalRot;
				moveState = ScreenMoveState.None;
			}
			else
			{
				screenTransform.localPosition = theNewPos;
				screenTransform.localRotation = theNewRot;
			}
		}
	}

	private void ToggleScreen()
	{
		screenIsUp = !screenIsUp;

		if( screenIsUp )
		{
			screenTransform.parent = inFrontOfCameraTransform;
			moveState = ScreenMoveState.TowardCamera;
			Screen.lockCursor = false;
			foreach( MouseLook theMouseLook in mouseLooks )
			{
				theMouseLook.enabled = false;
			}
		}
		else
		{
			screenTransform.parent = transform;
			moveState = ScreenMoveState.TowardOriginalPos;
			Screen.lockCursor = true;
			foreach( MouseLook theMouseLook in mouseLooks )
			{
				theMouseLook.enabled = true;
			}
		}
	}

	private void TouchScreen()
	{
		ScreenObject theClickedObject = HoveredScreenObject();
		if( theClickedObject )
		{
			theClickedObject.ToggleActivate();
		}
	}

	private void HoverScreen()
	{
		ScreenObject theHoveredObject = HoveredScreenObject();
		if( theHoveredObject )
		{
			theHoveredObject.UpdateHover();
		}
	}

	private ScreenObject HoveredScreenObject()
	{
		ScreenObject theHitObject = null;

		Vector3 theMousePos = Input.mousePosition;
		float theScreenDist = Vector3.Distance( Camera.main.transform.position, screenTransform.position );
		
		Vector3 theNearDistPos = Camera.main.ScreenToWorldPoint( new Vector3( theMousePos.x, theMousePos.y, Camera.main.nearClipPlane ) );
		Vector3 theScreenDistPos = Camera.main.ScreenToWorldPoint( new Vector3( theMousePos.x, theMousePos.y, theScreenDist ) );
		Vector3 theDelta = theScreenDistPos - theNearDistPos;
		
		RaycastHit theHit;
		if( Physics.Raycast( theNearDistPos, theDelta.normalized, out theHit, theDelta.magnitude ) )
		{
			if( theHit.collider.gameObject == screenObject )
			{
				Vector3 theRelScreenHitPos = screenTransform.InverseTransformPoint( theHit.point );

				float theXPercentage = ( theRelScreenHitPos.x + screenWidth / 2.0f ) / screenWidth;
				float theYPercentage = ( theRelScreenHitPos.y + screenHeight / 2.0f ) / screenHeight;

				Vector3 theScreenPixelCoords = new Vector3( screenCamera.targetTexture.width * theXPercentage,
				                                            screenCamera.targetTexture.height * theYPercentage,
				                                            screenCamera.nearClipPlane );
				//Debug.Log( "screen pixel coords: " + theScreenPixelCoords );

				Vector3 theHoveredWorldCoords = screenCamera.ScreenToWorldPoint( theScreenPixelCoords );

				theScreenPixelCoords.z = 100;
				Vector3 theSecond = screenCamera.ScreenToWorldPoint( theScreenPixelCoords );
				theDelta = theSecond - theHoveredWorldCoords;

				RaycastHit theScreenHit;
				if( Physics.Raycast( theHoveredWorldCoords, theDelta, out theScreenHit ) )
				{
					//Debug.Log( "hit object: " + theScreenHit.collider.gameObject );
					theHitObject = theScreenHit.collider.gameObject.GetComponent<ScreenObject>();
				}
			}
		}

		return theHitObject;
	}
}
